<TeXmacs|1.0.5.12>

<style|<tuple|mmxdoc|mathemagix>>

<\body>
  <\tmdoc-title>
    More about Mathemagix 2

    Make glue code for Mathemagix
  </tmdoc-title>

  <section|Make your first glue>

  To make a <em|glued code>, the first think to have is code to be glued.
  Here we take the simpliest example possible. Imagine you wrote a
  <verbatim|C++> function, working on <verbatim|int> (here I choose a
  function computing the fibonacci numbers called <verbatim|funnybunny>) and
  that you want to use it in Mathemagix.\ 

  Where you have the content of the file <verbatim|firstcode.hpp> that only
  contains the function <verbatim|funnybunny>:

  <\cpp-fragment>
    int

    funnybunny (int n) {\ 

    \ \ if (n == 0)\ 

    \ \ \ \ return 0;

    \ \ else {\ 

    \ \ \ \ if (n == 1) return 1;\ 

    \ \ \ \ else return funnybunny(n-1) + funnybunny(n-2);\ 

    \ \ }\ 

    }
  </cpp-fragment>

  To use this function in <value|mmx>, you need a to make description file
  explaining how to generate code that can be use by <value|mmx>. The \S
  discription files \T have the extension <verbatim|.mmh> and from this file
  (say <verbatim|myglue.mmh>), the program <verbatim|mmx-extend> generate two
  things:

  <\enumerate-numeric>
    <item>a file <verbatim|myglue.cpp> which is an \S object <verbatim|C++>
    \T file,

    <item>a file <verbatim|mmx-myglue.so> which is a static <verbatim|C++>
    object file that be called directly by <value|mmx> and generated from
    <verbatim|myglue.cpp>.
  </enumerate-numeric>

  So, the process can be describe as the generation of a <verbatim|C++> file
  (from a description file) that can be use to create an object file that can
  be linked dynamically by <value|mmx>. Here, we begin by giving the
  description file to use <verbatim|funnybunny> in <value|mmx> and we will
  detail the content of the file. Here after is the file
  <verbatim|firstglue.mmh>:

  <\cpp-fragment>
    preamble

    /"

    #include "firstcode.hpp"

    "/;

    \;

    export {

    \ \ funnybunny: Int -\<gtr\> Int == "funnybunny";

    }
  </cpp-fragment>

  To detail again the processus, using the command <verbatim|mmx-extend>: we
  generate from <verbatim|firstglue.mmh> a <verbatim|C++> file
  <verbatim|firstglue.cpp> which is compiled to obtain a file
  <verbatim|mmx-firstglue.so> wchich his a dynamically linkable object file
  that <value|mmx> can use if you copy it in the directory
  <verbatim|$LD_LIBRARY_PATH>.\ 

  The <verbatim|preamble> contains the included file, the new class and types
  (eventually) and some functions (we will see later what are those functions
  when there is some). \ 

  The code contained in the <verbatim|export> describe the how to use the
  code from your file in <value|mmx>. For instance, here we define an
  <value|mmx> function <verbatim|funnybunny> from the <verbatim|mmx:int> to
  the <verbatim|mmx:int> corresponding to the <verbatim|C++> code of
  <verbatim|fynnybunny> that has a corresponding signature.

  Now, you can launch <value|mmx>:

  <with|prog-language|mathemagix|prog-session|default|<\session>
    <\output>
      ------------------------------------------------------------

      \|:*) \ \ \ \ \ \ \ \ \ \ \ Welcome to Mathemagix 0.1.47
      \ \ \ \ \ \ \ \ \ \ \ (*:\|

      \| \ \ \ \ \ (c) 2001--2005 \ by Joris van der Hoeven et al.
      \ \ \ \ \ \|

      \| This software falls under the GNU General Public License \|

      \| \ \ \ \ \ \ \ \ It comes without any warranty whatsoever
      \ \ \ \ \ \ \ \ \|

      ------------------------------------------------------------
    </output>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      use "firstglue"
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      a : Int := int(6)
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      funnybunny(a)
    </input>

    <\output>
      <with|mode|math|8> <right-flush> <with|mode|math|color|brown|mode|text|Int>
    </output>
  </session>>

  The instruction <verbatim|use "file"> make <value|mmx> search the file
  <verbatim|mmx-file.so> in the directory containing the library.\ 

  We now explain a little how it works. As it is said above, the glue code
  file (the one with the extension <verbatim|.mmh>) is use by the command
  <verbatim|mmx-extend> in order to make a <verbatim|C++> file which is
  compatible with the <verbatim|C++> library used by <value|mmx>. This file
  is compiled using <verbatim|g++> in order to have a static object code file
  which can be dynamically load by the <value|mmx> interpreter. In order to
  allows the <verbatim|mmx-extend> command to generate the <verbatim|C++>
  code, together with your own code, you must provide some information on the
  tools you use from the usual <verbatim|C++> library used by your own code
  when it is use in <value|mmx>. You must also explain what are the
  functionalities of your code that you will use and how you will use it in
  <value|mmx>. Let us go more accuratly in this glue code:

  <\enumerate-numeric>
    <item>The first type of information is in the <verbatim|preamble>: for
    instance in <verbatim|firstglue.mml> we only precise that the
    <verbatim|C++> code used to make the static object code must include the
    file <verbatim|firstcode.hpp>. In the <verbatim|preamble> you only write
    <verbatim|C++> code.

    <item>The second type of information is in the <verbatim|export>: in the
    <verbatim|export> you say what are the functionalities of your own code
    that you will use in <value|mmx>. For instance, in our example you only
    say that you will use the function <verbatim|funnybunny> (the name of the
    <verbatim|C++> function is on the left and the name that will be used in
    <value|mmx> is on the right).
  </enumerate-numeric>

  <section|Second glue code : use your own <verbatim|C++> class in
  <value|mmx>>

  For the second glue, we propose a sloppy implementation of the Gauss
  integers to show how to do the glue code to use your hown types on
  <value|mmx>. Consider the following <verbatim|C++> class and functions
  (file <verbatim|secondcode.hpp>):

  Now, the corresponding glue code (file <verbatim|secondglue.mmh>):

  <\cpp-fragment>
    preamble

    /"

    #include "secondcode.hpp"

    \;

    std::ostream&

    print_lisp (std::ostream& out, const MyGaussInt& z) {

    \ \ out \<less\>\<less\> "(+ ";

    \ \ (void) print_lisp (out, real (z));

    \ \ out \<less\>\<less\> " (* ";

    \ \ (void) print_lisp (out, imag (z));

    \ \ out \<less\>\<less\> " mathi))";

    \ \ return out;

    }

    "/;

    \;

    export {

    \ \ Gaussian_int: Class == "MyGaussInt";

    \;

    \ \ new: Void -\<gtr\> Gaussian_int == "new";

    \ \ new: (Int, Int) -\<gtr\> Gaussian_int == "new";

    \ \ operator \<less\>\<less\>%: (OStream, Gaussian_int) -\<gtr\> OStream
    == "print_lisp";

    \ \ 

    \ \ real: Gaussian_int -\<gtr\> Int == "real";

    \ \ imag: Gaussian_int -\<gtr\> Int == "imag";

    \ \ 

    \ \ operator +: (Gaussian_int, Gaussian_int) -\<gtr\> Gaussian_int ==

    \ \ \ \ "operator +";

    \ \ operator -: (Gaussian_int, Gaussian_int) -\<gtr\> Gaussian_int ==

    \ \ \ \ "operator -";

    }
  </cpp-fragment>

  The following things are new :

  <\enumerate-numeric>
    <item>the declaration of the <value|mmx> type (here
    <with|color|red|<verbatim|GaussInt>> at the begining of the export),

    <item>the constructors corresponding to the new type,

    <item>the pretty-printing operator.
  </enumerate-numeric>

  <with|prog-language|mathemagix|prog-session|default|<\session>
    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      use "secondglue"
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      a : Int := int(7)
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      b : Int := int(9)
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      z : Gaussian_int := gaussian_int(a,b)
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      z
    </input>

    <\output>
      <with|mode|math|7+9*\<mathi\>> <right-flush>
      <with|mode|math|color|brown|<with|mode|text|Alias><left|(><with|mode|text|Gaussian_int><right|)>>
    </output>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      z := z+z+z+z
    </input>

    <\output>
      <with|mode|math|28+36*\<mathi\>> <right-flush>
      <with|mode|math|color|brown|mode|text|Gaussian_int>
    </output>
  </session>>

  We comment a little more accuratly the changes with the previous glue code:

  <\enumerate-numeric>
    <item>Firstly, we declare the function for the pretty-printing in the
    import in the form of a <verbatim|C++> function added there. If you
    provide a similar function (<verbatim|print_lisp>) for all the type you
    use you can also directly write the function in your code file and export
    it in the <verbatim|export> section.

    <item>In the <verbatim|export> section we add the decleration of our own
    type with the name that we will use for in <value|mmx>
    (<verbatim|GaussInt: Class == "MyGaussInt">), the constructors and the
    pretty-print function (the one declare in preamble). The rest of the code
    is similar to the one for functions.
  </enumerate-numeric>

  <section|Third glue code : glue generic codes>

  This section details an instance of glue code with your own templated
  <verbatim|C++> class. The code presented here is an evolution of the code
  shown in the previous section.\ 

  Let us first show the <verbatim|C++> code (the file is
  <verbatim|thirdcode.hpp>) :

  <\cpp-fragment>
    preamble

    /"

    #include "thirdcode.hpp"

    \;

    template\<less\>typename T\<gtr\> std::ostream&\ 

    print_lisp(std::ostream& out, const GaussLike\<less\>T\<gtr\>& z) {

    \ \ out \<less\>\<less\> "(+ ";

    \ \ (void) print_lisp(out, real(z));

    \ \ out \<less\>\<less\> " (* ";

    \ \ (void) print_lisp(out, imag(z));

    \ \ out \<less\>\<less\> " mathi))";

    \ \ return out;

    }

    "/;

    \;

    <\with|color|red>
      import {

      \ \ forall (R: Class) {

      \ \ \ \ new: Void -\<gtr\> R == "new";

      \ \ \ \ new: Int -\<gtr\> R == "new";

      \ \ \ \ operator +: (R, R) -\<gtr\> R == "operator +";

      \ \ \ \ operator -: (R, R) -\<gtr\> R == "operator -";

      \ \ \ \ operator *: (R, R) -\<gtr\> R == "operator *";

      \ \ \ \ operator \<less\>\<less\>%: (OStream, R) -\<gtr\> OStream ==
      "print_lisp";

      \ \ }

      }
    </with>

    \;

    export {

    \ \ <with|color|red|Complexified : Class -\<gtr\> Class == "GaussLike";>

    \;

    \ \ <with|color|red|forall (R: Class) \ {>

    \ \ \ \ new: Void -\<gtr\> Complexified R == "new";

    \ \ \ \ new: (R, R) -\<gtr\> Complexified R == "new";

    \ \ \ \ operator \<less\>\<less\>%: (OStream, Complexified R) -\<gtr\>
    OStream ==

    \ \ \ \ \ \ "print_lisp";

    \ \ 

    \ \ \ \ real: Complexified R -\<gtr\> R == "real";

    \ \ \ \ imag: Complexified R -\<gtr\> R == "imag";

    \ \ 

    \ \ \ \ operator +: (Complexified R, Complexified R) -\<gtr\>
    Complexified R

    \ \ \ \ \ \ == "operator +";

    \ \ \ \ operator -: (Complexified R, Complexified R) -\<gtr\>
    Complexified R

    \ \ \ \ \ \ == "operator -";

    \ \ \ \ operator *: (Complexified R, Complexified R) -\<gtr\>
    Complexified R

    \ \ \ \ \ \ == "operator *";

    \ \ <with|color|red|}>

    }
  </cpp-fragment>

  Globally, the class <verbatim|Complexified> is a generic class for
  <verbatim|MyGaussInt>. Below, you will find the corresponding glue code
  (the file <verbatim|thirdglue.mmh>) where we put in red the changes we made
  compare to the previous glue code :

  Then, if you use the <verbatim|mmx-extend> command on
  <verbatim|thirdglue.mmh> you obtain the file <verbatim|mmx-thirdglue.so>.
  If you copy this file in your library file, you can use the function just
  as below:

  <with|prog-language|mathemagix|prog-session|default|<\session>
    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      use "thirdglue"
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      Z : Complexified Integer \ := complexified(45,76)
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      Z
    </input>

    <\output>
      <with|mode|math|45+76*\<mathi\>> <right-flush>
      <with|mode|math|color|brown|<with|mode|text|Alias><left|(><with|mode|text|Complexified><left|(><with|mode|text|Integer><right|)><right|)>>
    </output>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      Y : Complexified Polynomial Integer :=
      complexified(polynomial(1,1),polynomial(2,3))
    </input>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      Y
    </input>

    <\output>
      <with|mode|math|x+1+<left|(>2*x+3<right|)>*\<mathi\>> <right-flush>
      <with|mode|math|color|brown|<with|mode|text|Alias><left|(><with|mode|text|Complexified><left|(><with|mode|text|Polynomial><left|(><with|mode|text|Integer><right|)><right|)><right|)>>
    </output>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      real(Y)
    </input>

    <\output>
      <with|mode|math|x+1> <right-flush> <with|mode|math|color|brown|<with|mode|text|Polynomial><left|(><with|mode|text|Integer><right|)>>
    </output>

    <\input|<with|color|red|Mmx <with|mode|math|\<gg\>>> >
      Y := Y * Y + Y * Y
    </input>

    <\output>
      <with|mode|math|\<um\>6*x<rsup|2>-20*x-16+<left|(>8*x<rsup|2>+20*x+12<right|)>*\<mathi\>>
      <right-flush> <with|mode|math|color|brown|<with|mode|text|Complexified><left|(><with|mode|text|Polynomial><left|(><with|mode|text|Integer><right|)><right|)>>
    </output>
  </session>>

  We detail a little more accuratly the changes occuring compared to the
  previous glue code :

  <\enumerate-numeric>
    <item>In the <verbatim|preamble> the only change is that the pretty print
    function is now a template function.

    <item>In the <verbatim|import> section, we import from <value|mmx> all
    the functions we will used in our templated code, for all types.

    <item>Most of the changes occure in the <verbatim|export> section: the
    definition of the types are at the begining of the section and the
    declaration of a templated type <verbatim|MyTemplatedType> is
    <verbatim|NameInMmx: Class -\<gtr\> Class == "MyTemplatedType">. All the
    rest of the function working on our templated types may be templated and
    then there are include in a <verbatim|forall (R: Class)> (several if
    there is several templated parameters).
  </enumerate-numeric>

  \;

  <tmdoc-copyright|2005|Joris van der Hoeven|Bernard Mourrain|Olivier Ruatta>

  <tmdoc-license|Permission is granted to copy, distribute and/or modify this
  document under the terms of the <hlink|GNU General Public
  License|http://www.gnu.org/licenses/gpl.txt>. If you don't have this file,
  write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.>
</body>