
/******************************************************************************
* MODULE     : compound_types.cc
* DESCRIPTION: Compound types for the Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extern.hh>
#include <single.hh>
#include <printer.hh>
#include <alias.hh>
namespace mmx {

/******************************************************************************
* Aliases
******************************************************************************/

generic
class_alias (generic g) {
  return list (ID_ALIAS, g[1]);
}

generic
alias_assign (generic g) {
  //cout << detailed << g[0][1] << " := " << g[0][2] << normal << lf;
  generic val= g[0][2];
  as_genalias (g[0][1])->set (val);
  return val;
}

generic
alias_assign_alias (generic g) {
  generic val= g[0][2];
  as_genalias (g[0][1])->set (as_genalias (val)->get ());
  return val;
}

/******************************************************************************
* Tuples and VTuples
******************************************************************************/

generic
class_tuple (generic g) {
  return list (ID_TUPLETYPE, g[1]);
}

generic
class_vtuple (generic g) {
  return list (ID_VTUPLETYPE, g[1]);
}

/******************************************************************************
* Generators
******************************************************************************/

generic
class_generator (generic g) {
  return list (ID_GENERATOR, g[1]);
}

/******************************************************************************
* Interface
******************************************************************************/

void
initialize_compound_types () {
  symbol T ("T");
  generic ID_TUPLET= list (ID_TUPLETYPE, T);
  generic ID_CLASS_CLASS= function_type (ID_CLASS, ID_CLASS);

  // aliases
  generic ID_ALIAST= list (ID_ALIAS, T);
  generic ID_ALIAST_T_T= function_type (ID_ALIAST, T, T);
  generic ID_QT_ALIAST_T_T= template_type (T, ID_ALIAST_T_T);
  declare (MMX_ALIAS, class_alias, ID_CLASS_CLASS);
  declare (MMX_ASSIGN, alias_assign, ID_QT_ALIAST_T_T);
  /* Problem when evaluating [1]
  generic ID_ALIAST_ALIAST_ALIAST=
    function_type (ID_ALIAST, ID_ALIAST, ID_ALIAST);
  generic ID_QT_ALIAST_ALIAST_ALIAST=
    template_type (T, ID_ALIAST_ALIAST_ALIAST);
  declare (MMX_ASSIGN, alias_assign_alias, ID_QT_ALIAST_ALIAST_ALIAST);
  */

  // tuples and vtuples
  declare (MMX_TUPLETYPE, class_tuple, ID_CLASS_CLASS);
  declare (MMX_VTUPLETYPE, class_vtuple, ID_CLASS_CLASS);

  // generators
  declare (MMX_GENERATOR, class_generator, ID_CLASS_CLASS);
}

} // namespace mmx
