/******************************************************************************
* MODULE     : mmx_shell.cc
* DESCRIPTION: Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <list.hh>
#include <file.hh>
#include <error.hh>
#include <parser.hh>
#include <evaluator.hh>
#include <pretty.hh>
#include <shell.hh>
#include <fstream>
#include <iostream>

using namespace mmx;
GC_GLOBAL_STATIC (int, line_counter, 0);

static void
error_report (generic g) {
  list errs= cdr (as_list (g));
  print_errors (errs);
}

static void
output (generic g) {
  cout << blank;
  if (is_value (g)) mmx_display (as_value (g));
  else cout << g;
  cout << stroke;
}

static void
evaluate(string& s, list& errors, generic& g, bool interactive) {
    // Evaluate input
    errors= list ();
    g= parse (s, errors);
    if (N(errors)!=0) {
      print_errors (errors);
      return;
    }
    g= mmx_evaluate (g);
    if (is_func (g, symbol ("errors!"))) error_report (g);
    else if (interactive && (g != VAL_NONE) && !quiet_mode) output (g);
};

int
main (int argc, char** argv) {

  GC_LOCAL (bool, interactive, false);
  GC_LOCAL (int, arg_nr, 0);
  GC_LOCAL (string, s, "");
  GC_LOCAL (string, arg, "");
  GC_LOCAL (list, errors, list ());
  GC_LOCAL (list, to_load, list ());
  GC_LOCAL (generic, g, generic ());

  mmeval= mmx::evaluator (true);
  initialize_master ();
  to_load = shell_initialize(argc, argv);

  if (load_boot_file) {
    if (load (".:$MMX_LOAD_PATH", "boot.mmx", s)) {
      cout << "Couldn't open boot file boot.mmx\n";
    }
    else {
      evaluate(s, errors, g, interactive);
    }
  }
    
  while(!nil(to_load)) {
    arg= as_string(car(to_load));
    to_load=cdr(to_load);
    if (load (arg, s)) {
      cout << "Couldn't open file " << arg << "\n";
      continue;
    }
    cout << "Processing " << arg << "\n";
    evaluate(s, errors, g, interactive);
  }

  while (true) {
    if (texmacs_mode)
      s= shell_texmacs_input (line_counter);
    else
      s= shell_terminal_input (line_counter);
    if ((s == "quit") || (s == "quit;")) break;
    if ((s == "exit") || (s == "exit;")) break;
    if ((s == "bye") || (s == "bye;")) break;
    if ((s == "") || (s == ";")) continue;
    if ((s[N(s)-1] != ';') && (s[N(s)-1] != '}')) s= s * ";";
    insert_source (s, "cmdline");
    interactive= true;
    //gc_debug_flag= true;
    evaluate(s, errors, g, interactive);
  }
  exit(0);
}
