/******************************************************************************
* MODULE     : init.cc
* DESCRIPTION: mathemagix shell initialization functions
* COPYRIGHT  : (C) 2005  Gregoire Lecerf
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <shell.hh>
#include <evaluator.hh>

namespace mmx {
using std::cout;
using std::cin;

GC_GLOBAL (bool, load_boot_file, true);
GC_GLOBAL (bool, completion_mode, true);
GC_GLOBAL (bool, quiet_mode, false);

static void
help(string progname) {
  cerr << "Usage: " << progname << " [-b on|off] [-c on|off] [-q] [--version] [--debug] [--texmacs] [--help] [filelist]\n";
  cerr << "  -b on|off\t" << "enable|disable loading boot file; default is on.\n";
  cerr << "  -c on|off\t" << "enable|disable the completion mode; default is on.\n";
  cerr << "  -q on|off\t" << "quiet mode: do not display prompts and output; default is off.\n";
  cerr << "  --version\t" << "display version number.\n";
  cerr << "  --debug\t" << "enable debug mode.\n";
  cerr << "  --texmacs\t" << "enable texmacs mode; should be enabled by texmacs only.\n";
  cerr << "  --help\t" << "display this message.\n";
}

static list
handle_cmdline_options(int argc, char** argv) {

  int c;
  int option_index = 0;
  string arg, tmp;
  list to_load;
  string progname = string(argv[0]);

  opterr=0;
  struct option long_options[] =
    {
      {"version", 0, 0, 0},
      {"debug", 0, 0, 0},
      {"texmacs", 0, 0, 0},
      {"help", 0, 0, 0},
      {0, 0, 0, 0}
    };

  while (1) {
    option_index = 0;

    c = getopt_long (argc, argv, "b:c:q:",
		     long_options, &option_index);
    if (c == -1)
      break;

    if (optarg)
      arg = string(optarg);

    switch (c)
      {
      case 0:
	tmp = string((char*)long_options[option_index].name);
	insert_source (tmp, "cmdline");	
	if (tmp=="texmacs") {
	  texmacs_mode=true;
	  break;
	}
	if (tmp=="debug") {
	  debug_output=true;
	  break;
	}
	if (tmp=="version") {
	  cout << VERSION << "\n";
	  exit(0);
	}

	if (tmp=="help") {
	  help(progname);
	  exit(0);
	}

	cerr << progname << " error in command line\n";
	help(progname);
	exit(-1);
      
      case 'b':
	tmp = "-b " * arg;
	insert_source (tmp, "cmdline");	
	if (arg=="off") {
	  load_boot_file=false;
	  break;
	}
	if (arg=="on") {
	  load_boot_file=true;
	  break;
	}
	cerr << progname << " error in command line\n";
	help(progname);
	exit(-1);
	
      case 'c':
	tmp = "-c " * arg; 
	insert_source (tmp, "cmdline");	
	if (arg=="off") {
	  completion_mode=false;
	  break;
	}
	if (arg=="on") {
	  completion_mode=true;
	  break;
	}
	cerr << progname << " error in command line\n";
	help(progname);
	exit(-1);
	
      case 'q':
	tmp = "-q";
	insert_source (tmp, "cmdline");	
	if (arg=="off") {
	  quiet_mode=false;
	  break;
	}
	if (arg=="on") {
	  quiet_mode=true;
	  break;
	}
	cerr << progname << " error in command line\n";
	help(progname);
	exit(-1);
	
      case '?':
	cerr << progname << " : bad command line\n";
	help(progname);
	exit(-1);
	
      default:
	cerr << progname << " : error in command line\n";
      }
  }
  
  if (optind < argc) {
    while (optind < argc) 
      to_load = cons(string(argv[optind++]), to_load);
  }
  return to_load;
}

static void
output_banner(void) {
  cout << "------------------------------------------------------------\n";
  cout << "|:*)            Welcome to Mathemagix " << VERSION
       << "            (*:|\n";
  cout << "|      (c) 2001--2005  by Joris van der Hoeven et al.      |\n";
  cout << "| This software falls under the GNU General Public License |\n";
  cout << "|         It comes without any warranty whatsoever         |\n";
  cout << "------------------------------------------------------------\n";
}

list
shell_initialize(int argc, char** argv) {
  list to_load;
  to_load = handle_cmdline_options(argc, argv);
  if (texmacs_mode)
    shell_texmacs_initialize();
  if (!quiet_mode) output_banner();
  return to_load;
}

} // namespace mmx
