
/******************************************************************************
* MODULE     : converter.cc
* DESCRIPTION: converters of type expressions
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <converter.hh>
#include <single.hh>
namespace mmx {

/******************************************************************************
* Constructing the converter
******************************************************************************/

converter_rep::converter_rep () {
  dcv= list (ID_AND);
  lcv= list (ID_AND);
  rcv= list (ID_AND);
}

void
converter_rep::add (generic what, bool left_ass, bool right_ass) {
  dcv= as_list (dcv) * list (what);
  if (left_ass)  lcv= as_list (lcv) * list (what);
  if (right_ass) rcv= as_list (rcv) * list (what);
  cout << dcv << lf;
}

/******************************************************************************
* Required routines for generic structures
******************************************************************************/

size_t
converter_rep::Size () {
  return sizeof (converter_rep);
}

void
converter_rep::Mark () {
  mark (dcv);
  mark (lcv);
  mark (rcv);
}

int
converter_rep::Type () {
  return CONVERTER_TYPE;
}

int
converter_rep::Hash () {
  return (int) ((long) ((void*) this));
}

bool
converter_rep::Equal (generic g) {
  if (gentype (g) != CONVERTER_TYPE) return false;
  return converter (this) == as_converter (g);
}

generic
converter_rep::Copy () {
  return copy (converter (this));
}

int
converter_rep::Arity () {
  return 0;
}

void
converter_rep::Print (ostream& out) {
  out << converter (this);
}

/******************************************************************************
* Standard routines for converters
******************************************************************************/

ostream&
operator << (ostream& out, converter cv) {
  if (current_print_format == indented) {
    out << "#converter";
    current_indentation++;
    out << lf << cv->dcv;
    current_indentation--;
    return out;
  }
  else return out << "#converter<" << cv->dcv << ">";
}

converter
copy (converter cv) {
  converter cv2;
  cv2->dcv= copy (as_list (cv->dcv));
  cv2->lcv= copy (as_list (cv->lcv));
  cv2->rcv= copy (as_list (cv->rcv));
  return cv2;
}

} // namespace mmx
