
/******************************************************************************
* MODULE     : xtable.hh
* DESCRIPTION: extended tables (for local environments)
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __XTABLE_HH
#define __XTABLE_HH
#include <gentable.hh>
#include <table.hh>
namespace mmx {

class xtable;
class xtable_rep: public gentable_rep {
public:
  gentable base_t;
  gentable ext_t;

protected:
  size_t Size ();
  void Mark ();
  int Type ();
  int Hash ();
  generic Copy ();
  void Print (ostream& out);

public:
  inline xtable_rep (gentable base_t2, gentable ext_t2):
    base_t (base_t2), ext_t (ext_t2) {}

  generic  get_default ();
  generic  get_binding (generic key);
  void     set_binding (generic binding);
  void     reset (generic key);
  list     entries ();
  bool     local_contains (generic key);
  generic  local_read (generic key);
  generic& local_write (generic key);
  int      local_level ();

  friend class xtable;
  friend xtable copy (xtable t);
  friend ostream& operator << (ostream& out, xtable t);
};

class xtable {
DEFAULT_METHODS(xtable);
public:
  inline xtable (gentable base_t): rep (new xtable_rep (base_t, table ())) {}
  inline xtable (gentable base_t, gentable ext_t):
    rep (new xtable_rep (base_t, ext_t)) {}
  inline operator gentable () { return gentable (rep); }

  inline generic  operator [] (generic g) { return rep->read (g); }
  inline generic& operator () (generic g) { return rep->write (g); }
  inline friend bool operator == (xtable a, xtable b) { return a->Equal (b); }
  inline friend bool operator != (xtable a, xtable b) { return !a->Equal (b); }

  friend xtable copy (xtable a);
  friend ostream& operator << (ostream& out, xtable a);
};

void recursive_dump (gentable t, int threshold=10);

}
#endif // __XTABLE_HH
